@@ -1,5 +1,5 @@
-# Copyright (C) 2003 by the gtk2-perl team (see the file AUTHORS for the full
-# list)
+# Copyright (C) 2003-2004 by the gtk2-perl team (see the file AUTHORS for
+# the full list)
 # 
 # This library is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Library General Public License as published by the Free
@@ -15,7 +15,7 @@
 # along with this library; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307  USA.
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/Glib.pm,v 1.52 2004/01/30 01:03:47 rwmcfa1 Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/Glib.pm,v 1.61 2004/03/02 05:37:36 muppetman Exp $
 #
 
 package Glib;
@@ -35,6 +35,7 @@ use constant {
 	G_PRIORITY_HIGH_IDLE    =>  100,
 	G_PRIORITY_DEFAULT_IDLE =>  200,
 	G_PRIORITY_LOW	        =>  300,
+	G_PARAM_READWRITE       => [qw/readable writable/],
 };
 
 # export nothing by default.
@@ -48,6 +49,7 @@ our %EXPORT_TAGS = (
 			G_PRIORITY_HIGH_IDLE
 			G_PRIORITY_DEFAULT_IDLE
 			G_PRIORITY_LOW
+			G_PARAM_READWRITE
 			/],
 	functions => [qw/
 			filename_to_unicode
@@ -59,7 +61,7 @@ our %EXPORT_TAGS = (
 our @EXPORT_OK = map { @$_ } values %EXPORT_TAGS;
 $EXPORT_TAGS{all} = \@EXPORT_OK;
 
-our $VERSION = '1.033';
+our $VERSION = '1.038';
 
 sub dl_load_flags { $^O eq 'darwin' ? 0x00 : 0x01 }
 
@@ -78,7 +80,19 @@ use overload
    '@{}'  => \&as_arrayref,
    '""'   => sub { "[ @{$_[0]} ]" },
    fallback => 1;
-   
+
+package Glib::Error;
+
+use overload
+   '""' => sub { $_[0]->message.$_[0]->location },
+   fallback => 1;
+
+sub location { $_[0]->{location} }
+sub message { $_[0]->{message} }
+sub domain { $_[0]->{domain} }
+sub value { $_[0]->{value} }
+sub code { $_[0]->{code} }
+
 package Glib::Object::Property;
 
 use Carp;
@@ -240,8 +254,9 @@ be allowed (e.g. GtkTreeModel columns):
  Glib::Scalar
 
 Functions that can return false and set a GError in C raise an exception in
-Perl (using the string from the GError for $@).  Exceptions are a sticky issue,
-so they get their own section.
+Perl, using an exception object based on the GError for $@; see L<Glib::Error>.
+Trapping exceptions in signals is a sticky issue, so they get their own
+section; see L<EXCEPTIONS>.
 
 Enumerations and flags are treated as strings and arrays of strings,
 respectively.  GLib provides a way to register nicknames for enumeration
@@ -430,6 +445,7 @@ you can also get all of them with the export tag "all".
   G_PRIORITY_HIGH_IDLE
   G_PRIORITY_DEFAULT_IDLE
   G_PRIORITY_LOW
+  G_PARAM_READWRITE
 
 =item Tag: functions
 
@@ -490,7 +506,7 @@ tools.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003 by muppet and the gtk2-perl team
+Copyright 2003-2004 by muppet and the gtk2-perl team
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the Lesser General Public License (LGPL).  For 
