@@ -1,10 +1,10 @@
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/MakeHelper.pm,v 1.23 2004/01/27 15:46:09 rwmcfa1 Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/MakeHelper.pm,v 1.26 2004/02/25 20:21:22 rwmcfa1 Exp $
 #
 
 package Glib::MakeHelper;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 =head1 NAME
 
@@ -54,6 +54,7 @@ the string --disable-apidoc an empty list will be returned and thus no apidoc
 pod will be generated speeding up the build process.
 
 =cut
+
 sub do_pod_files
 {
 	return () if (grep /disable[-_]apidoc/i, @ARGV);
@@ -99,6 +100,7 @@ or 'realclean' keys is that this avoids you having to remember to put Glib's
 stuff in your Makefile.PL's WriteMakefile arguments.
 
 =cut
+
 sub postamble_clean
 {
 	shift; # package name
@@ -110,6 +112,8 @@ realclean ::
 
 =item string = Glib::MakeHelper->postamble_docs (@xs_files)
 
+NOTE: this is The Old Way.  see L<postamble_docs_full> for The New Way.
+
 Create and return the text of Makefile rules to build documentation from
 the XS files with Glib::ParseXSDoc and Glib::GenPod.
 
@@ -127,10 +131,139 @@ anywhere a rule needs to be sure that a loadable and working module resides in
 the blib directory before running.
 
 =cut
+
 sub postamble_docs
 {
-	shift; # package name
-	my @xs_files = @_;
+	my ($class, @xs_files) = @_;
+	return Glib::MakeHelper->postamble_docs_full (XS_FILES => \@xs_files);
+}
+
+=item string = Glib::MakeHelper->postamble_docs_full (...)
+
+Create and return the text of Makefile rules to build documentation from
+the XS files with Glib::ParseXSDoc and Glib::GenPod.
+
+Use this in your MY::postamble to enable autogeneration of POD.
+
+This updates dependencies with the list of pod names generated by an earlier
+run of C<do_pod_files>.
+
+There is a special Makefile variable POD_DEPENDS that should be set to the
+list of files that need to be created before the doc.pl step is run, include
+files.
+
+There is also a variable BLIB_DONE which should be used as a dependancy
+anywhere a rule needs to be sure that a loadable and working module resides in
+the blib directory before running.
+
+The parameters are a list of key=>value pairs.  You must specify at minimum
+either DEPENDS or XS_FILES.
+
+=over
+
+=item DEPENDS => ExtUtils::Depends object
+
+Most gtk2-perl modules use ExtUtils::Depends to find headers, typemaps,
+and other data from parent modules and to install this data for child
+modules.  We can find from this object the list of XS files to scan for
+documentation, doctype mappings for parent modules, and other goodies.
+
+=item XS_FILES => \@xs_file_names
+
+A list of xs files to scan for documentation.  Ignored if DEPENDS is
+used.
+
+=item DOCTYPES => \@doctypes_file_names
+
+List of filenames to pass to C<Glib::GenPod::add_types>.  May be omitted.
+
+=item COPYRIGHT => string
+
+POD text to be inserted in the 'COPYRIGHT' section of each generated page.
+May be omitted.
+
+=item COPYRIGHT_FROM => file name
+
+The name of a file containing the POD to be inserted in the 'COPYRIGHT'
+section of each generated page.  May be omitted.
+
+=back
+
+=cut
+
+sub postamble_docs_full {
+	my $class = shift; # package name
+	my %params = @_;
+
+	croak "Usage: $class\->postamble_docs_full (...)\n"
+	    . "  where ... is a list of key/value pairs including at the\n"
+	    . "  very least one of DEPENDS=>\$extutils_depends_object or\n"
+	    . "  XS_FILES=>\@xs_files\n"
+	    . "    "
+		unless $params{DEPENDS} or $params{XS_FILES};
+
+	my @xs_files = ();
+	my @doctypes = ();
+	my $add_types = '';
+	my $copyright = '';
+
+	if ($params{DOCTYPES}) {
+		@doctypes = ('ARRAY' eq ref $params{DOCTYPES})
+		          ? @{$params{DOCTYPES}}
+		          : ($params{DOCTYPES});
+	}
+
+	if (UNIVERSAL::isa ($params{DEPENDS}, 'ExtUtils::Depends')) {
+		my $dep = $params{DEPENDS};
+
+		# fetch list of XS files from depends object
+		@xs_files = @{$dep->{xs}};
+
+		# fetch doctypes files from the depends' dependencies.
+		my %deps = $dep->get_deps;
+		foreach my $d (keys %deps) {
+			my $f = File::Spec->catfile ($deps{$d}{instpath},
+			                             'doctypes');
+			#warn "looking for $f\n";
+			push @doctypes, $f
+				if -f $f;
+		}
+	} else {
+		@xs_files = @{ $params{XS_FILES} };
+	}
+
+	if ($params{COPYRIGHT}) {
+		$copyright = $params{COPYRIGHT};
+	} elsif ($params{COPYRIGHT_FROM}) {
+		open IN, $params{COPYRIGHT_FROM} or
+			croak "can't open $params{COPYRIGHT_FROM} for reading: $!\n";
+		local $/ = undef;
+		$copyright = <IN>;
+		close IN;
+	}
+
+	if ($copyright) {
+		# this text has to be escaped for both make and the shell.
+		$copyright =~ s/\n/\\n/gm; # collapse to one line.
+		$copyright =~ s/"/\"/gm;   # escape double-quotes
+		$copyright = "\$\$Glib::GenPod::COPYRIGHT=\"$copyright\";";
+	}
+
+	#warn "".scalar(@doctypes)." doctype files\n";
+	#warn "".scalar(@xs_files)." xs files\n";
+	
+	$add_types = "add_types (".join(", ",map {"\"$_\""} @doctypes)."); "
+		if @doctypes;
+
+	my $docgen_code = ''
+	    . $add_types
+	    . ' '
+	    . $copyright
+	    . ' $(POD_SET) '
+	    . 'xsdoc2pod("build/doc.pl", "$(INST_LIB)", "build/podindex");';
+
+	#warn "docgen_code: $docgen_code\n";
+
 "
 
 # BLIB_DONE should be set to something we can depend on that will ensure that
@@ -157,7 +290,7 @@ build/doc.pl :: Makefile @xs_files
 
 build/podindex :: \$(BLIB_DONE) Makefile build/doc.pl
 	$^X -I \$(INST_LIB) -I \$(INST_ARCHLIB) -MGlib::GenPod -M\$(NAME) \\
-		-e \"\$(POD_SET) xsdoc2pod('build/doc.pl', '\$(INST_LIB)', 'build/podindex');\"
+		-e '$docgen_code'
 
 \$(INST_LIB)/\$(FULLEXT)/:
 	mkdir -p \$@
@@ -192,6 +325,7 @@ the build setups for Glib and Gtk2 for examples.
 Note: This function just returns an empty string on Win32.
 
 =cut
+
 sub postamble_rpms
 {
 	shift; # package name
@@ -239,6 +373,8 @@ dist-srpms :: Makefile dist perl-\$(DISTNAME).spec \$(RPMS_DIR)/
 
 package MY;
 
+=back
+
 =head1 NOTICE
 
 The MakeMaker distributed with perl 5.8.x generates makefiles with a bug that
@@ -270,4 +406,12 @@ Ross McFarland E<lt>rwmcfa1 at neces dot comE<gt>
 
 hacked up and documented by muppet.
 
+=head1 COPYRIGHT AND LICENSE
+
+Copyright 2003-2004 by the gtk2-perl team
+
+This library is free software; you can redistribute it and/or modify
+it under the terms of the Lesser General Public License (LGPL).  For 
+more information, see http://www.fsf.org/licenses/lgpl.txt
+
 =cut
